;===========================================================================================
; TreeView_1.asm  Fasm Flat assembler
; Controle des dossiers sous forme d'arbre
; Programm par AsmGges France
;===========================================================================================

format PE GUI 4.0
entry start

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  include 'win32a.inc'
  include 'macro\if.inc'

;===========================================================================================
; Dbut du code programme
; -----------------------
section '.code' code readable executable

start:

    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    or eax,eax                              ;tester si eax=0
    jz ErrorModule                          ;flag processeur z=0 => erreur
    mov [wc.hInstance],eax                  ;handle du programme

    ; Chargement d'une icne
    ; ----------------------
    invoke LoadIcon,[wc.hInstance],IDI_FRANCE
    or eax,eax                              ;tester si eax=0
    jz ErrorIcon                            ;flag processeur z=0 => erreur
    mov [wc.hIcon],eax                      ;handle de l'icone

    ; Chargement d'un curseur
    ; -----------------------
    invoke LoadCursor,0,IDC_ARROW
    or eax,eax                              ;tester si eax=0
    jz ErrorCursor                          ;flag processeur z=0 => erreur
    mov [wc.hCursor],eax                    ;handle du curseur

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,[WinW]                          ;soustraire largeur fenetre de largeur cran
    shr eax,1                               ;diviser le reste par 2
    mov [WinX],eax                          ;rsultat = coordonne X gauche de la fenetre

    ; Obtenir la hauteur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CYSCREEN
    sub eax,[WinH]                          ;soustraire hauteur fenetre de hauteur cran
    shr eax,1                               ;diviser le reste par 2
    mov [WinY],eax                          ;rsultat = coordonne Y haute de la fenetre

    ; Valider le choix de la structure WNDCLASS
    ; -----------------------------------------
    invoke RegisterClass,wc
    or eax,eax                              ;tester si eax=0
    jz ErrorCrWin                           ;flag processeur z=0 => erreur

    ; Cration de la fenetre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,0100h,ClassName,Title,04CA0000h,[WinX],[WinY],[WinW],[WinH],\
                          0,0,[wc.hInstance],0
    or eax,eax                              ;tester si eax=0
    jz ErrorCrWin                           ;flag processeur z=0 => erreur
    mov [hWnd],eax                          ;handle de la fenetre

    ; Chargement de la bibliothque de liaison dynamique de contrle commune (DLL)
    ; ----------------------------------------------------------------------------
    invoke InitCommonControls

    ; Affichage de la fenetre
    ; -----------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT

    ; Rafraichissement de la fenetre
    ; ------------------------------
    invoke UpdateWindow,[hWnd]

 ;----------------------------------
 ; Boucle de traitement des messages
 ;----------------------------------
 MessageLoop:

    ; Rceptionne les messages
    ; ------------------------
    invoke GetMessage,msg,0,0,0
    or eax,eax                              ;tester si eax=0
    jz ExitProgram                          ;flag processeur z=0 => quitter l'application

    ; Traduit les touches d'entre clavier en messages
    ; ------------------------------------------------
    invoke TranslateMessage,msg

    ; Expdition des messages  la procdure WindowProc
    ; -------------------------------------------------
    invoke DispatchMessage,msg

    jmp MessageLoop                         ;bouclage du traitement des messages

 ;---------------------------------
 ; Traitement des messages d'erreur
 ;---------------------------------
 ErrorModule:
    mov eax,MsgErrModule                    ;GetModuleHandle erreur
    jmp Error
 ErrorIcon:
    mov eax,MsgErrIcon                      ;LoadIcon erreur
    jmp Error
 ErrorCursor:
    mov eax,MsgErrCursor                    ;LoadCursor erreur
    jmp Error
 ErrorCrWin:
    mov eax,MsgErrCrWin                     ;CreateWindowEx erreur
 Error:
    ; Boite  message
    ; ---------------
    invoke MessageBox,[hWnd],eax,ClassName,MB_ICONHAND

    ; Dtruit la fenetre
    ; ------------------
    invoke DestroyWindow,[hWnd]

    ; Demande la fin de l'application
    ; -------------------------------
    invoke PostQuitMessage,1

    jmp MessageLoop                         ;retour dans la boucle des messages

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[wc.hInstance]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
; @hWnd   = handle de la fenetre qui reoit les messages
; @uMsg   = le numro du message
; @wParam = renseignements supplmentaires du message
; @lParam = renseignements supplmentaires du message
;-------------------------------------------------------------------------------------------
proc WindowProc uses ebx esi edi, @hWnd, @uMsg, @wParam, @lParam

    .if [@uMsg] = WM_CLOSE
        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],MsgEnd,ClassName,MB_OK+MB_ICONINFORMATION

        ; Demande la fin de l'application
        ; ------------------------------
        invoke PostQuitMessage,0

    .elseif [@uMsg] = WM_CREATE
        ; Cration d'une fenetre
        ; ----------------------
        invoke CreateWindowEx,0300h,TreeViewClass,0,054000005h,10,10,200,250,[@hWnd],\
                              0500h,[wc.hInstance],0
        mov [hTreeV],eax                ;handle de la nouvelle fenetre

        ; Cre une nouvelle liste d'image
        ; -------------------------------
        invoke ImageList_Create,16,16,ILC_COLOR16 + ILC_MASK,2,10      ; avec masque
        mov [hImgl],eax                 ;handle de la liste d'image

        ; Chargement en mmoire d'un bitmap
        ; ---------------------------------
        invoke LoadBitmap,[wc.hInstance],IDB_DOSSIER
        mov [hBitmap],eax               ;handle du bitmap

        ; Ajoute une ou des images  une liste d'image
        ; gnrant un masque de bitmap indiqu
        ; --------------------------------------------
        invoke ImageList_AddMasked,[hImgl],[hBitmap],00FF00h            ; avec masque

        ; Supprime un objet et son contexte de dispositif
        ; -----------------------------------------------
        invoke DeleteObject,[hBitmap]

        ; Envoie un message  une ou des fentres
        ; ---------------------------------------
        invoke SendMessage,[hTreeV],TVM_SETIMAGELIST,TVSIL_NORMAL,[hImgl]

        mov [tvinsert.hParent],NULL
        mov [tvinsert.hInsertAfter],TVI_ROOT
        mov [tvinsert.item.mask],TVIF_TEXT+TVIF_IMAGE+TVIF_SELECTEDIMAGE
        mov [tvinsert.item.pszText],TreeVTxt1
        mov [tvinsert.item.iImage],0
        mov [tvinsert.item.iSelectedImage],1

        ; Envoie un message  une ou des fentres
        ; ---------------------------------------
        invoke SendMessage,[hTreeV],TVM_INSERTITEM,0,tvinsert
        mov [tvinsert.hParent],eax

        mov [tvinsert.hInsertAfter],TVI_LAST
        mov [tvinsert.item.pszText],TreeVTxt2

        ; Envoie un message  une ou des fentres
        ; ---------------------------------------
        invoke SendMessage,[hTreeV],TVM_INSERTITEM,0,tvinsert

        mov  [tvinsert.item.pszText],TreeVTxt3

        ; Envoie un message  une ou des fentres
        ; ---------------------------------------
        invoke SendMessage,[hTreeV],TVM_INSERTITEM,0,tvinsert

    .elseif [@uMsg] = WM_NOTIFY
        mov  edx,[@lParam]
        .if [edx + NMTREEVIEW.hdr.code] = TVN_BEGINDRAG
            ; Envoie un message  une ou des fentres
            ; ---------------------------------------
            invoke SendMessage,[hTreeV],TVM_CREATEDRAGIMAGE,0,\
                               [edx + NMTREEVIEW.itemNew.hItem]
            mov [hImglDrag],eax

            ; Envoie un message  une ou des fentres
            ; ---------------------------------------
            invoke SendMessage,[hTreeV],TVM_GETITEMRECT,TRUE,imglitemrect

            ; Commence  faire glisser une image
            ; ----------------------------------
            invoke ImageList_BeginDrag,[hImglDrag],0,0,0

            ; Ferme les mises  jour  la fentre indique pendant
            ; une opration de glisser et montre l'image qui
            ; glisse  la position indique dans la fentre.
            ; ----------------------------------------------------
            invoke ImageList_DragEnter,[hTreeV],[edx + NMTREEVIEW.ptDrag.x],\
                                       [edx + NMTREEVIEW.ptDrag.y]

            ; Rcupre le handle de la fentre d'enfant de la fentre parent
            ; --------------------------------------------------------------
            invoke GetParent,[hTreeV]

            ; Capture la souris
            ; -----------------
            invoke SetCapture,eax
            mov [ImglDrag],TRUE

        .endif

    .elseif [@uMsg] = WM_MOUSEMOVE
        .if [ImglDrag] = TRUE
            mov eax,[@lParam]
            and eax,0xFFFF
            mov ecx,[@lParam]
            shr ecx,16
            mov [tvhitinfo.pt.x],eax
            mov [tvhitinfo.pt.y],ecx

            ; Dplace l'image pendant une opration de glisser-dplacer
            ; Utilise en rponse  un message WM_MOUSEMOVE.
            ; ---------------------------------------------------------
            invoke ImageList_DragMove,eax,ecx

            ; Montre ou cache l'image glisse
            ; -------------------------------
            invoke ImageList_DragShowNolock,FALSE

            ; Envoie un message  une ou des fentres
            ; ---------------------------------------
            invoke SendMessage,[hTreeV],TVM_HITTEST,0,tvhitinfo

            .if eax <> NULL
                ; Envoie un message  une ou des fentres
                ; ---------------------------------------
                invoke SendMessage,[hTreeV],TVM_SELECTITEM,TVGN_DROPHILITE,eax

            .endif
            ; Montre ou cache l'image glisse
            ; -------------------------------
            invoke ImageList_DragShowNolock,TRUE

        .endif

    .elseif [@uMsg] = WM_LBUTTONUP
        .if [ImglDrag] = TRUE
            ; Ouvre la fentre indique et cache l'image qui
            ; glisse permettant  la fentre d'tre mis  jour
            ; ------------------------------------------------
            invoke ImageList_DragLeave,[hTreeV]

            ; Fin d'une opration de glisser
            ; ------------------------------
            invoke ImageList_EndDrag

            ; Envoie un message  une ou des fentres
            ; ---------------------------------------
            invoke SendMessage,[hTreeV],TVM_GETNEXTITEM,TVGN_DROPHILITE,0

            ; Envoie un message  une ou des fentres
            ; ---------------------------------------
            invoke SendMessage,[hTreeV],TVM_SELECTITEM,TVGN_CARET,eax

            ; Envoie un message  une ou des fentres
            ; ---------------------------------------
            invoke SendMessage,[hTreeV],TVM_SELECTITEM,TVGN_DROPHILITE,NULL

            ; Fin capture souris
            ; ------------------
            invoke ReleaseCapture
            mov [ImglDrag],FALSE

        .endif

    .else
        ; Traitement par dfaut des autres messages Windows
        ; -------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .endif

    mov eax,0
    ret
endp

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
section '.data' data readable writeable

  Title         db ' TreeView',0                  ;titre de la fenetre
  ClassName     db ' AsmGges Win32',0             ;classe de la fenetre
  MsgEnd        db ' AsmGges France @2006  ',0
  TreeViewClass db 'SysTreeView32',0              ;classe de la fenetre Tree

  ; Texte Tree
  TreeVTxt1     db 'Node - Parent',0
  TreeVTxt2     db 'Node - Child 1',0
  TreeVTxt3     db 'Node - Child 2',0

  ; Messages d'erreur
  ; -----------------
  MsgErrModule db 'GetModuleHandle: erreur',0
  MsgErrIcon   db 'LoadIcon: erreur',0
  MsgErrCursor db 'LoadCursor: erreur',0
  MsgErrCrWin  db 'CreateWindowEx: erreur',0

  align 4                     ;aprs les chaines sur 8bits alignement des datas sur 32bits

  WinX       dd 0             ;coordonne X relative gauche de la fenetre
  WinY       dd 0             ;coordonne Y relative haute de la fenetre
  WinW       dd 230           ;coordonne X relative droite de la fenetre
  WinH       dd 300           ;coordonne Y relative basse de la fenetre

  ImglDrag   dd FALSE

  hWnd       dd ?             ;handle de la fenetre
  hBitmap    dd ?             ;handle du bitmap
  hImgl      dd ?             ;handle de la liste d'image
  hImglDrag  dd ?
  hTreeV     dd ?             ;handle de la nouvelle fenetre

  wc           WNDCLASS  0,WindowProc,0,0,NULL,NULL,NULL,COLOR_BTNFACE+1,NULL,ClassName
  msg          MSG            ;structure pour les messages de Windows
  imglitemrect RECT           ;structure pour coordonnes oprations dans la fenetre
  tvinsert     TVINSERTSTRUCT ;structure pour item
  tvhitinfo    TVHITTESTINFO  ;structure pour item

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------
section '.idata' import data readable writeable

  library kernel32,'KERNEL32.DLL',\
            user32,'USER32.DLL',\
             gdi32,'GDI32.DLL',\
          comctl32,'COMCTL32.DLL'

  ; Apis Windows dans fichiers dlls
  ; -------------------------------
  include 'apia\kernel32.inc'
  include 'apia\user32.inc'
  include 'apia\gdi32.inc'
  include 'apia\comctl32.inc'

;===========================================================================================
; Inclusion des icnes / curseurs / bitmap
;-------------------------------------------------------------------------------------------
section '.rsrc' resource data readable

  directory RT_ICON,icons,\
            RT_GROUP_ICON,group_icons,\
            RT_BITMAP,appBmp

  IDI_FRANCE  = 17                              ;id de l'icone
  IDB_DOSSIER = 301                             ;id du bitmap

  resource icons,\
           1,LANG_NEUTRAL,icon_data

  resource group_icons,\
           IDI_FRANCE,LANG_NEUTRAL,main_icon

  resource appBmp,\
           IDB_DOSSIER,LANG_NEUTRAL,bmp1

  bitmap bmp1,'res\dossier.bmp'              ;chemin fichier du bitmap

  icon main_icon,icon_data,'res\france.ico'  ;chemin fichier de l'icone

;===========================================================================================
